#ifndef lint
static char SccsId[] = "%W%  %G%";
#endif

/* Module:	clrmenu.c (Color Menu)
 * Purpose:	Read and Write file and internally stored color tables
 * Subroutine:	fetch_colortable()		returns:  int
 * Subroutine:  new_color_table_download()	returns:  int
 * Subroutine:	read_color_file			returns:  ColorTable static
 * Subroutine:  write_color_file		returns:  void static
 * Subroutine:  print_one_color			returns:  void static
 * Subroutine:	load_newtable			returns:  void static
 * Subroutine:	load_subtable			returns:  void static
 * Xlib calls:	none
 * Copyright:	1989 Smithsonian Astrophysical Observatory
 *		You may do anything you like with this file except remove
 *		this copyright.  The Smithsonian Astrophysical Observatory
 *		makes no representations about the suitability of this
 *		software for any purpose.  It is provided "as is" without
 *		express or implied warranty.
 * Modified:	{0} Michael VanHilst	initial version		  16 May 1989
 *		{1} Donald H. Gudehus;  registered c_map to f_map    Sep 1993
 *		{2} Donald H. Gudehus;  introduced new subroutine 
 *		new_color_table_download to download custom LUTs via the
 *		IMTOOL protocol.				 21 Oct., 1994
 *		{3} Donald H. Gudehus;  changed code for sscanf section in
 *		new_color_table_download so that MIPS RISC compiler could
 * 		produce functioning code.			 25, Nov., 1994
 *		{4} Donald H. Gudehus;  added (Enhanced Version) to 
 *		SAOimage label.					 16, Aug., 1999
 *		{n} <who> -- <does what> -- <when>
 */

#include <stdio.h>			/* stderr, NULL, etc. */
#include <X11/Xlib.h>			/* X window stuff */
#include <X11/Xutil.h>			/* X window manager stuff */
#include "hfiles/constant.h"		/* code values */
#include "hfiles/colormap.h"		/* color table storage structs */
#include "hfiles/define.h"		/* define SZ_FNAME, etc. */
#include "hfiles/edit.h"		
#include "hfiles/imtool.h"		/* to get value of SZ_FIFOBUF */
#include "hfiles/struct.h"		/* color structs in color.h, 
					windowRec for graphbox in window.h,
					ConnectRec in control.h */
#include "hfiles/extern.h"		/* for color
/* EditStruct */
#include "defs/colormap.def"		/* define color tables */

EditStruct *color_edit;	/* key to popup editor for file name input */

/*
 * Subroutine:	fetch_colortable
 * Purpose:	Gets a new pre-defined color table and load it for use
 * Returns:	1 if succeeded, else 0
 */
int fetch_colortable ( color, table_code, imagefile )
     struct colorRec *color;
     int table_code;
     char *imagefile;		/* i: imagefile name to print in output file */
{
  int j;
  ColorTable *newtable;
  static void load_newtable(), write_color_file();
  static ColorTable *read_color_file();

  switch( table_code ) {
  case MOP_Init_A:
    newtable = &gray_map;
    break;
  case MOP_Init_B:
    newtable = &bbs_map;
    break;
  case MOP_Init_C:
    newtable = &hip_map;
    break;
  case MOP_Init_D:
    newtable = &imp8_map;
    break;
  case MOP_Init_E:
    newtable = &a_map;
    break;
  case MOP_Init_F:
    newtable = &b_map;
    break;
  case MOP_Init_G:
    newtable = &c_map;
    break;
  case MOP_Init_H:
    newtable = &d_map;
    break;
  case MOP_Init_I:
    newtable = &e_map;
    break;
  case MOP_Init_J:
    newtable = &f_map;
    break;
  case MOP_Init_K:
    newtable = &g_map;
    break;
  case MOP_Read:
    if( (newtable = read_color_file()) == NULL )
      return( 0 );
    break;
  case MOP_Write:
    write_color_file(imagefile, color);
    return( 0 );
  default:
    (void)fprintf(stderr, "WARNING: unknown color map code!\n");
    return( 0 );
  }
  load_newtable(&color->ctable, newtable);
  return( 1 );
}

static ColorTable ctable;

/*
 * Subroutine:  new_color_table_download
 * Purpose:     Installs a new color table down-loaded via the IMTOOL
 *              protocol.  Returns the remaining number of bytes to be read
 */
int new_color_table_download (port, ndatabytes)
  struct connectRec *port;   /* i:  struct of port for Imtool input */
  int ndatabytes;            /* i:  number of bytes in expected message */
     
{
  int bytes, j, jj;
  ColorTable newt, *newtable;
  char ibuf[SZ_FIFOBUF];     /* working array */ 
  float a,b;

  int read_connection();
  static void load_newtable();
  void make_cellstore_from_tables(), draw_cgraph(), label_gamma();
  void set_submenu_toggle();

  newtable = &newt;
  bytes = read_connection(port, ibuf, ndatabytes);
  ndatabytes -= bytes;
  sscanf(ibuf, "%d%d%f", &newtable->red.vertex_cnt,
  &newtable->red.do_gamma, &newtable->red.gamma);
  if (newtable->red.vertex_cnt != 200) {
    (void) fprintf(stderr, "Vertex count must be equal to 200\n");
    return ( ndatabytes);
  }
  sscanf(&ibuf[2413], "%d%d%f", &newtable->green.vertex_cnt,
  &newtable->green.do_gamma, &newtable->green.gamma);
  sscanf(&ibuf[4826], "%d%d%f", &newtable->blue.vertex_cnt,
  &newtable->blue.do_gamma, &newtable->blue.gamma);
  jj = 0;
/*  The variables a and b are used below in an intermediate step so that
    the MIPS RISC compiler could function.  With VAX C this problem doesn't
    exist, and newtable->xxx can be used in the sscanf statement.  */
  for (j = 0; j < 2400 ; j = j +12) {
    sscanf(&ibuf[13 + j], "%f%f", &a, &b);
    newtable->red.cell_level[jj] = a;
    newtable->red.intensity[jj] = b;
    sscanf(&ibuf[2426 + j], "%f%f", &a, &b);
    newtable->green.cell_level[jj] = a;
    newtable->green.intensity[jj] = b; 
    sscanf(&ibuf[4839 + j], "%f%f", &a, &b);
    newtable->blue.cell_level[jj] = a;
    newtable->blue.intensity[jj] = b; 
    jj = jj + 1;
  }
/*  printf(" #1 red green blue %d  %d  %d\n", newtable->red.vertex_cnt, 
  newtable->green.vertex_cnt, newtable->blue.vertex_cnt);
  for (j = 0; j < 200; ++j) {
  printf(" red level intensity %d %f   %f\n",j,newtable->red.cell_level[j],
  newtable->red.intensity[j]);
  printf(" green level intensity %f   %f\n",newtable->green.cell_level[j],
  newtable->green.intensity[j]);
  printf(" blue level intensity %f   %f\n\n",newtable->blue.cell_level[j],
  newtable->blue.intensity[j]);
  } */
  load_newtable(&color.ctable, newtable);
  /* unset the inverse flag if it is set */
  if (color.inverse) {
    set_submenu_toggle(VOP, VOP_Invert, 0);
    color.inverse = 0;
  }
   make_cellstore_from_tables(&color);
  /* send colors to the display */
  XStoreColors(color.display, color.colormap,
               color.cellstore, color.ncolors);
  /* draw the graph of the colors, and relabel the gamma values */
  if( graphbox.active ) {
    draw_cgraph(1, 0);
    label_gamma(1, 1, 1);
  }
  return ( ndatabytes);
}

/*
 * Subroutine:	read_color_file
 * Purpose:	Opens and reads a color table file, and load it into the
 *		current color table.
 */
static ColorTable *read_color_file ( )
{
  FILE *fp;
  int open_input_file(), parse_color_file();
  EditStruct *init_edit_popup();

  if( color_edit == NULL )
    color_edit = init_edit_popup((char *)NULL, SZ_FNAME);
  /* open coordinate output file for writing */
  if( open_input_file(&fp, color_edit, 0,
		       "Enter name of pseudocolor file:") <= 0 )
    return( NULL );
  bzero((char *)&ctable, sizeof(ColorTable));
  if( parse_color_file(fp, &ctable, CTBL_MAX) ) {
    (void)fclose(fp);
    return( &ctable );
  } else {
    (void)fclose(fp);
    return( NULL );
  }
}

/*
 * Subroutine: write_color_file
 * Purpose:    Opens and writes a color table file
 */
static void write_color_file ( imagefile, color )
     char *imagefile;
     struct colorRec *color;
{
  FILE *fp;
  int error;
  static void print_one_color();
  EditStruct *init_edit_popup();
  int open_output_file();
  void timestamp();

  error = 0;
  if( color->ctable.red.vertex_cnt > CTBL_MAX ) {
    (void)fprintf(stderr,
		  "WARNING: too many red entries for portable table\n");
    error = 1;
  }
  if( color->ctable.green.vertex_cnt > CTBL_MAX ) {
    (void)fprintf(stderr,
		  "WARNING: too many green entries for portable table\n");
    error = 1;
  }
  if( color->ctable.blue.vertex_cnt > CTBL_MAX ) {
    (void)fprintf(stderr,
		  "WARNING: too many blue entries for portable table\n");
    error = 1;
  }
  if( color_edit == NULL )
    color_edit = init_edit_popup((char *)NULL, SZ_FNAME);
  /* open coordinate output file for writing */
  if( open_output_file(&fp, color_edit, 0,
		      "Enter name of pseudocolor file:") <= 0 )
    error = 1;
  if( error ) {
    (void)fprintf(stderr,"No color file written\n");
    XBell(color->display, 80);
    return;
  }
  /* timestamp the first entry in the output file. */
  (void)fprintf(fp, "# SAOimage (Enhanced Version) color table\n");
  timestamp(fp, imagefile);
  (void)fprintf(fp, "PSEUDOCOLOR\n");
  (void)fprintf(fp, "RED:");
  print_one_color(fp, &color->ctable.red);
  (void)fprintf(fp, "GREEN:");
  print_one_color(fp, &color->ctable.green);
  (void)fprintf(fp, "BLUE:");
  print_one_color(fp, &color->ctable.blue);
  fclose(fp);
}

/*
 * Subroutine:  print_one_color
 * Purpose:	Prints on color
 */
static void print_one_color ( fp, table )
     FILE *fp;
     struct subtableRec *table;
{
  int i, j;
  if( table->do_gamma )
    (void)fprintf(fp, " gamma %.3f\n", table->gamma);
  else
    (void)fprintf(fp, "\n");
  for( i=0, j=0; i<table->vertex_cnt; i++ ) {
    (void)fprintf(fp,"(%.3f,%.3f)",table->cell_level[i],table->intensity[i]);
    if( ++j >= 5 ) {
      j=0;
      (void)fprintf(fp, "\n");
    }
  }
  (void)fprintf(fp, "\n");
}

/*
 * Subroutine:	load_newtable
 * Purpose:	Loads an internally stored color table for use
 */
static void load_newtable ( ctable, new )
     struct colorTable *ctable;
     ColorTable *new;
{
  static void load_subtable();

  load_subtable(&ctable->red, &new->red);
  load_subtable(&ctable->green, &new->green);
  load_subtable(&ctable->blue, &new->blue);
}

/*
 * Subroutine:	load_subtable
 * Purpose:	Loads one color of the internal color table
 */
static void load_subtable ( subtable, new )
     struct subtableRec *subtable;
     struct SubTable *new;
{
  int i;

  subtable->fixed_cells = 0;
  if( new->do_gamma ) {
    subtable->do_gamma = 1;
    subtable->gamma = new->gamma;
  } else {
    subtable->do_gamma = 0;
    subtable->gamma = 1.0;
  }
  subtable->invert_order = 0;
  subtable->vertex_cnt = new->vertex_cnt;
  for( i=0; i<new->vertex_cnt; i++ ) {
    subtable->cell_level[i] = new->cell_level[i];
    /* base levels used for dynamic manipulation */
    subtable->base_level[i] = new->cell_level[i] - 0.5;
    subtable->intensity[i] = new->intensity[i];
  }
  subtable->bias = 0.5;
  subtable->contrast = 1.0;
}
