#ifndef lint
static char SccsId[] = "%W%  %G%";
#endif

/* Module:	ctrlsckt.c (Control Socket)
 * Purpose:	Open a pipe device for IO
 * Subroutine:	open_socket_listener()		returns: int
 * Subroutine:	accept_socket_connection()	returns: struct *connectRec
 * Subroutine:	close_socket()			returns: void
 * Subroutine:	flush_socket()			returns: void
 * Copyright:	1990 Smithsonian Astrophysical Observatory
 *		You may do anything you like with this file except remove
 *		this copyright.  The Smithsonian Astrophysical Observatory
 *		makes no representations about the suitability of this
 *		software for any purpose.  It is provided "as is" without
 *		express or implied warranty.
 * Modified:	{0} John Roll	initial version			 1 March 1990
 *		{1} MVH simplified for new connect module	10 March 1990
 *		{2} MVH corrected htonl bug			14 June 1993
 *              {3} MVH modified to support INET & UNIX sockets 20 Nov 1993
 *		{n} <who> -- <does what> -- <when>
 */

#ifndef VMS

#include <stdio.h>
#include <sys/types.h>
#include <sys/socket.h>		/* socket(), bind(), listen(), accept(), etc */
#include <netdb.h>
#include <netinet/in.h>		/* htonl(), struct sockaddr_in */
#ifdef SYSV
#include <string.h>		/* strlen(), strcpy(), strncpy(), strrchr() */
#else
#include <strings.h>		/* strlen(), strcpy(), strncpy(), rindex() */
#endif

#include <X11/Xlib.h>		/* X window stuff */
#include "hfiles/control.h"	/* define struct connectRec */

/* %% PUT THESE IN CONTROL.H */
#define IOP_socket_listener 777
#define IOP_socket_acceptee 555


/*
 * Subroutine:  open_socket_listener
 * Purpose:     Get a UNIX socket on which to listen for connections
 */
#ifdef __STDC__
int open_socket_listener ( struct connectRec *listener )
#else
int open_socket_listener ( listener )
     struct connectRec *listener;
#endif
{
  if( listener->subtype == IOP_inet )
    return open_inet_socket_listener(&(listener->name), listener->address);
  else if( listener->subtype == IOP_unix )
    return open_unix_socket_listener(listener->name);
}


/*
 * Subroutine:	accept_socket_connection
 * Purpose:	accept a connection requested of the socket listener
 */
#ifdef __STDC__
struct connectRec *accept_socket_connection ( struct connectRec *listener )
#else
struct connectRec *accept_socket_connection ( listener )
     struct connectRec *listener;
#endif
{
  struct sockaddr net_address;
  int ipc;
  int address_len;
  struct connectRec *connection;
  char *calloc_errchk();

  address_len = sizeof(net_address);
  if( (ipc = accept(listener->fd, &net_address, &address_len)) < 0 ) {
    perror("Cannot accept socket connection");
    return( NULL );
  }
  connection = (struct connectRec *)
   calloc_errchk(1, sizeof(struct connectRec), "socket descriptor");
  connection->fd = ipc;
  connection->type = IOP_socket;
  connection->direction = listener->direction;
  connection->protocol = listener->protocol;
  connection->func = listener->func;
  connection->name = listener->name;
  connection->address = listener->address;
  /*  link this connection record with the original listener record  */
  /*  IF YOU PLAN TO ALLOW MULTIPLE ACCEPTS PER LISTENER, DO MORE HERE  */
  connection->affiliate = listener;
  listener->affiliate = connection;
  return( connection );
}

/*
 * Subroutine:	flush_socket
 * Purpose:	flush all pending input on this socket connection
 */
#ifdef __STDC__
void flush_socket ( int ipc, char* socketname )
#else
void flush_socket ( ipc, socketname )
     int ipc;
     char* socketname;
#endif
{
  extern void flush_disk();

  if( socketname == NULL )
    flush_disk(ipc, "socket");
  else
    flush_disk(ipc, socketname);
}


/*
 * Subroutine:	close_socket_connection
 * Purpose:	close this socket's file descriptor
 *		and undo any file system stuff that needs cleaning up
 */
#ifdef __STDC__
void close_socket_connection ( struct connectRec* connection )
#else
void close_socket_connection ( connection )
     struct connectRec* connection;
#endif
{
  if( connection->type == IOP_socket_listener ) {
    if( connection->subtype == IOP_inet )
      close_inet_socket(connection->fd, connection->name);
    else if( connection->subtype == IOP_unix )
      close_unix_socket(connection->fd, connection->name);
  } else {
    if( close(connection->fd) )
      perror("Error closing socket\n  ");
  }
}

#endif

