#ifndef lint
static char SccsId[] = "%W%  %G%";
#endif

/* Module:	ctrluskt.c (Control UNIX domain Socket)
 * Purpose:	Open a PF_UNIX address socket device for IO
 * Subroutine:	open_unix_socket_listener()	returns: int
 * Subroutine:	close_unix_socket()		returns: void
 * Copyright:	1990 Smithsonian Astrophysical Observatory
 *		You may do anything you like with this file except remove
 *		this copyright.  The Smithsonian Astrophysical Observatory
 *		makes no representations about the suitability of this
 *		software for any purpose.  It is provided "as is" without
 *		express or implied warranty.
 * Modified:	{0} MVH	initial version			 	20 Nov 1993
 *		{1} Donald H. Gudehus;  added (Enhanced Version) to
 *		SAOimage label.					16, Aug., 1999
 *		{n} <who> -- <does what> -- <when>
 */

#ifndef VMS

#include <stdio.h>
#include <sys/types.h>
#include <sys/socket.h>		/* socket(), bind(), listen(), accept(), etc */
#include <sys/time.h>
#include <sys/un.h>
#include <sys/stat.h>
#include <unistd.h>
#include <pwd.h>
#ifdef SYSV
#include <string.h>		/* strlen(), strcpy(), strncpy(), strrchr() */
#else
#include <strings.h>		/* strlen(), strcpy(), strncpy(), rindex() */
#endif

#include <X11/Xlib.h>		/* X window stuff */
#include "hfiles/control.h"	/* define struct connectRec */

#ifdef __STDC__
static int expand_user_filename(char*, char*);
static int socket_already_exists(struct sockaddr_un*,int);
#else
static int expand_user_filename();
static int socket_already_exists();
#endif



/*
 * Subroutine:	open_unix_socket_listener
 * Purpose:     Get a UNIX socket on which to listen for connections
 *		create the file entry needed for making connections
 */
#ifdef __STDC__
int open_unix_socket_listener ( char* file_name )
#else
int open_unix_socket_listener ( file_name )
     char* file_name;	/* i: filename to use (in user home dir if no path) */
#endif
{
  int sock, length;
  struct sockaddr_un name;
  char path_name[128];

  /*  Create socket  */
  sock = socket(AF_UNIX, SOCK_STREAM, 0);
  if( sock < 0 ) {
    perror("opening stream socket");
    return( -1 );
  }
  /*  Name socket with file-type name in user's login directory  */
  name.sun_family = AF_UNIX;
  if( expand_user_filename(file_name, path_name) ) {
    close(sock);
    return( -1 );
  }
  strcpy(name.sun_path, path_name);
  if( socket_already_exists(&name, sock) ) {
    return( -1 );
  }
  if( bind(sock, (struct sockaddr*)(&name), sizeof(struct sockaddr_un)) ) {
    (void)fprintf(stderr, "%s\n", name.sun_path);
    perror("binding name to stream socket");
    close(sock);
    return( -1 );
  }
  /*  Start accepting connections (allow a queue of up to 5 clients)  */
  listen(sock, 5);
  return( sock );
}


/*
 * Subroutine:	close_socket_listener
 * Purpose:	close this socket's file descriptor
 *		and undo the file system entry
 */
#ifdef __STDC__
void close_unix_socket ( int sock, char* socketname )
#else
void close_unix_socket ( sock, socketname )
     int sock;
     char* socketname;
#endif
{
  /*   Remove the socket name from the file system  */
  if( unlink(socketname) < 0 )
    perror("setting unlink");
  close(sock);
}


/*
 * Subroutine:	expand_user_filename
 * Purpose:	support function for UNIX sockets
 *		get full path name to put filename in user home directory
 * Returns:	-1 on failure, else 0
 */
#ifdef __STDC__
static int expand_user_filename ( char* file_name, char* path_name )
#else
static int expand_user_filename ( file_name, path_name )
     char* file_name;	/* i: file name to use  */
     char* path_name;	/* o: returned full path name  */
#endif
{
  struct passwd *psswd;
  char *login;
  uid_t uid;

  /*  if file name includes a path, don't bother doing this */
  if( file_name[0] == '/' ) {
    strcpy(path_name, file_name);
    return( 0 );
  }
  /*  add user's home directory path  */
  if( (login = getlogin()) == NULL ) {
    if( (uid = getuid()) == NULL ) {
      perror("Can't identify user");
      return( -1 );
    }
    if( (psswd = getpwuid(uid)) == NULL ) {
      perror("Can't get user directory from user id");
      return( -1 );
    }
  } else {
    if( (psswd = getpwnam(login)) == NULL ) {
      perror("Can't get user directory from user name");
      return( -1 );
    }
  }
  /*  Assemble the full path name  */
  strcpy(path_name, psswd->pw_dir);
  strcat(path_name, "/");
  strcat(path_name, file_name);
  return( 0 );
}


/*
 * Subroutine:	socket_already_exists
 * Purpose:	support function for UNIX sockets
 *		check if named socket is already in use
 * Returns:	0 if OK to open a new socket, -1 if socket exists and is in use
 */
#ifdef __STDC__
static int socket_already_exists ( struct sockaddr_un* name, int sock )
#else
static int socket_already_exists ( name, sock )
     struct sockaddr_un* name;	/* struct for name we want to use */
     int sock;			/* an unbound socket */
#endif
{
  struct stat st;
  char buf[1024];
  int rval;

  /*  Make sure name doesn't already exist  */
  if( lstat(name->sun_path, &st) == 0 ) {
    /*  Found file!  see if it is a live server.  */
    if( connect(sock, (struct sockaddr*)name, sizeof(struct sockaddr_un))
       < 0 ) {
      /*  Not taking connections, OK for us to become the server.  */
      /*  Make system call to remove the old socket entry.  */
      strcpy(buf, "rm ");
      strcat(buf, name->sun_path);
      system(buf);
    } else {
      /*  It's there and it's alive!  */
      (void)printf(
      "SAOimage (Enhanced Version) socket already has a server!\n");
      /*  Since we've established a connection, act like a client  */
      /*  If connection to another SAOimage generates any traffic, *
       *  we should clean things up before disconnecting 	   */
      do {
        bzero(buf, sizeof(buf));
        if( (rval = read(sock, buf, 1024)) < 0 )
          perror("reading stream message");
#ifdef DEBUG
        if( rval == 0 )
          (void)printf("Ending connection\n");
        else
          (void)printf("Current server message --> %s\n", buf);
#endif
      } while( rval != 0 );
      /*  Now exit gracefully  */
      close(sock);
      return( -1 );
    }
  }
  return( 0 );
}


#ifdef CLIENT
/* NOTE: This is the code for the client side */

/*
 * Subroutine:	open_unix_socket_connection
 * Purpose:	Get a socket connected to the listener
 */
#ifdef __STDC__
static int open_unix_socket_connection ( char** socket_name )
#else
static int open_unix_socket_connection ( socket_name )
     char** socket_name;
#endif
{
  int sock;
  struct sockaddr_un name;

  name.sun_family = AF_UNIX;
  /*  If no path, socket uses a file name in user's login directory  */
  if( expand_user_filename(file_name, name.sun_path) )
    return( -1 );
  /*  Create socket  */
  if( (sock = socket(AF_UNIX, SOCK_STREAM, 0)) < 0 ) {
    perror("opening stream socket");
    return( -1 );
  }
  /*  Establish connection with listener  */
  if( connect(sock, (struct sockaddr*)&name, sizeof struct sockaddr_un) <0 ) {
    perror("connecting stream socket");
    close(sock);
    return( -1 );
  }
  return( sock );
}
#endif

#endif

