#ifndef lint
static char SccsId[] = "%W%  %G%";
#endif

/* Module:	drawvect.c (Draw Vectors)
 * Purpose:  To draw cursor objects and vectors onto the desktop and into
 * the image buffer.  Called by do_overlay_from_imtool.
 * Subroutine:	draw_on_frame returns:  void
 * Subroutine:  vect_on_frame returns:  void
 * Subroutine:  expand_byte_to_short returns:  void
 * Modified:	{0} Donald H. Gudehus;  			August, 1993
 *		{1} Donald H. Gudehus;  added argument 'key' to vect_on_frame
 *		and created new subroutine display_vect_on_frame to make it 
 *		easier to draw single vectors                September, 1994
 *		{n} <who> -- <does what> -- <when>
 */

#ifdef IMTOOL

#include <stdio.h>		/* stderr, FILE, NULL, etc. */
#include <X11/Xlib.h>		/* get X types and constants */
#include <X11/Xutil.h>		/* X window manager stuff */
#include "hfiles/constant.h"	/* define codes */
#include "hfiles/struct.h"	/* declare structure types */
#include "hfiles/define.h"	/* for MAX and MIN */
#include "hfiles/extern.h"	/* extern main ximage parameter structures */
#include "hfiles/imtool.h"
#include <math.h>


/*
 * Subroutine:	draw_on_frame()
 * Purpose:  This subroutine draws various cursor types into the frame buffer
 * memory.  An entire pixel is lit up, and the cursor object is not
 * erasable unless the frame is erased.  Currently, a rectangle, circle, 
 * and ellipse are supported.  The ellipse and rectangle do not respond to 
 * the inclination at present.  This subroutine is used for designating file
 * areas that have been edited.
 */
void draw_on_frame (cursor_type, color_type, file_X, file_Y, a, b, inclina)
  int cursor_type,color_type;
  float file_X, file_Y, a, b, inclina;
{
  union {
    unsigned char readbuf[SZ_FIFOBUF];
    float fbuf[SZ_FIFOBUF/4];
  } u ;
  short *imagebuf, *startbuf;
  float xfb, yfb, xfc, yfc, delta_angle;
  int bytes, i, x1, x2, y1, y2, x, y, nvals_x, nvals_y, numpairs;
  void disp_subpiece();
  static void expand_byte_to_short();
  void d_transform(), vect_on_frame();

/* Put the coordinates in terms of the frame buffer system */
  d_transform( &coord.filetoimg, (double)file_X, (double)file_Y, &xfb, &yfb);
  if (cursor_type == 768) {  /* Rectangle */
    x1 = xfb - a;  /* Follows the convention used in MIIPS to produce
			rectangles with both odd and even sides. */
    x2 = xfb + a;
    y2 = yfb + b;  /* File and frame y coordinates are opposite */
    y1 = yfb - b;
/* In spite of the name, img.filecols refers to the horizontal size of the
   frame buffer */
    x1 = MAX(0, MIN(img.filecols - 1, x1));
    x2 = MAX(0, MIN(img.filecols - 1, x2));
    y1 = MAX(0, MIN(img.filerows - 1, y1));
    y2 = MAX(0, MIN(img.filerows - 1, y2));
/* Draw the sides of the rectangle in the following order:  top, bottom, 
   left, right.  In memory, we always increase */
    nvals_x = x2 - x1 + 1;
    nvals_y = y2 - y1 + 1;
    for (i = 0; i < MAX(nvals_x, nvals_y); i++)
      u.readbuf[i] = color_type;
/* Top */
    startbuf = (short *)buffer.filebuf + (y1 * img.filecols) + x1;
    expand_byte_to_short ((unsigned char *)u.readbuf, startbuf, nvals_x);
/* Bottom */
    startbuf = (short *)buffer.filebuf + (y2 * img.filecols) + x1;
    expand_byte_to_short ((unsigned char *)u.readbuf, startbuf, nvals_x);
/* Left side */
    for (i = 0; i < nvals_y; i++) {
      startbuf = (short *)buffer.filebuf + ((y1 + i) * img.filecols) + x1;
      expand_byte_to_short ((unsigned char *)u.readbuf, startbuf, 1);
    }
/* Right side */
    for (i = 0; i < nvals_y; i++) {
      startbuf = (short *)buffer.filebuf + ((y1 + i) * img.filecols) + x2;
      expand_byte_to_short ((unsigned char *)u.readbuf, startbuf, 1);
    }
    disp_subpiece(0, y1, img.filecols - 1, y2);

  } else if (cursor_type == 1024) {  /* circle */
/* Make the number of vectors describing the circle increase as the radius
   increases */
    numpairs = MIN(RND(a) + 1, 45);  /* numpairs is for a quadrant.  In any
		case, we must always have numpairs < SZ_FIFOBUF/32 */
    delta_angle = PI/(2 * numpairs);
/* Rely upon vect_on_frame to do limit checking */
/* First quadrant */
    u.fbuf[0] = a;
    u.fbuf[1] = 0.0;
    for (i = 2; i < numpairs * 2; i = i + 2) {
      u.fbuf[i] = a * cos((double)( delta_angle * i / 2));
      u.fbuf[i +1] = sqrt((double)(pow((double)a, 2.0) - 
                                   pow((double)u.fbuf[i], 2.0))) * b / a;
    }
/* Second quadrant */
    u.fbuf[numpairs * 2] = 0.0;
    u.fbuf[numpairs * 2 + 1] = b;
    for (i = 2; i < numpairs * 2; i = i + 2) {
      u.fbuf[numpairs * 2 + i] = -u.fbuf[numpairs * 2 - i];
      u.fbuf[numpairs * 2 + i +1] = u.fbuf[numpairs * 2 - i + 1];
    }
/* Third quadrant */
    u.fbuf[numpairs * 4] = -a;
    u.fbuf[numpairs * 4 + 1] = 0.0;
    for (i = 2; i < numpairs * 2; i = i + 2) {
      u.fbuf[numpairs * 4 + i] = -u.fbuf[i];
      u.fbuf[numpairs * 4 + i +1] = -u.fbuf[i + 1];
    }
/* Fourth quadrant */
    u.fbuf[numpairs * 6] = 0.0;
    u.fbuf[numpairs * 6 + 1] = -b;
    for (i = 2; i < numpairs * 2; i = i + 2) {
      u.fbuf[numpairs * 6 + i] = u.fbuf[numpairs * 2 - i];
      u.fbuf[numpairs * 6 + i +1] = -u.fbuf[numpairs * 2 - i + 1];
    }
/* Finish point */
    u.fbuf[numpairs * 8] = a;
    u.fbuf[numpairs * 8 + 1] = 0.0;
/* Draw the circle */
    for (i = 0; i < numpairs * 8; i = i + 2) {
      vect_on_frame(color_type, u.fbuf[i] + file_X, u.fbuf[i + 1] + file_Y,
      u.fbuf[i + 2] + file_X, u.fbuf[i + 3] + file_Y, 0);
    }
/*  y1 must be less than y2;  must do limit checking.  */
    y2 = MAX(0, MIN(img.filerows - 1, yfb + b));
/*  Make the y1 limit more generous by 1 pixel since the top of the
    circle is not always caught */
    y1 = MAX(0, MIN(img.filerows - 1, yfb - b - 1));
    disp_subpiece(0, y1, img.filecols - 1, y2);
  }
}

/*
 * Subroutine:	vect_on_frame()
 * Purpose:  This subroutine draws a vector in the frame buffer from one 
 * point to another.  It is designed to produce symmetric looking circles 
 * when used for that purpose.  If key = 0, the calling routine must call 
 * disp_subpiece after all the vectors have been drawn in order to have 
 * them displayed;  if key = 1, then this subroutine takes care of the
 * work.
 */
void vect_on_frame (color_type, file_x1, file_y1, file_x2, file_y2, key)
  float file_x1, file_y1, file_x2, file_y2;
  int color_type, key;
{
  unsigned char readbuf[SZ_FIFOBUF];
  short *imagebuf, *startbuf;
  float fbx1, fby1, fbx2, fby2, ratio, delta;
  int bytes, i, x1, y1, x2, y2, nvals_x, nvals_y, xstart, ystart;
  void disp_subpiece();
  static void expand_byte_to_short();
  void d_transform();

/* Put the coordinates in terms of the frame buffer system.  Also apply
   a nudge to the y coordinate so that the correct pixel will be chosen
   when at a pixel boundary. */
  file_y1 = file_y1 + 0.0001;
  file_y2 = file_y2 + 0.0001;
  d_transform( &coord.filetoimg, (double)file_x1, (double)file_y1,
  &fbx1, &fby1);
  d_transform( &coord.filetoimg, (double)file_x2, (double)file_y2,
  &fbx2, &fby2);
/* Convert to integer values for dealing with memory indexes */
  x1 = MAX(0, MIN(img.filecols - 1, fbx1));
  x2 = MAX(0, MIN(img.filecols - 1, fbx2));
  y1 = MAX(0, MIN(img.filerows - 1, fby1));
  y2 = MAX(0, MIN(img.filerows - 1, fby2));
  nvals_x = abs(x2 - x1) + 1;
  nvals_y = abs(y2 - y1) + 1;
  for (i = 0; i < MAX(nvals_x, nvals_y); i++)
    readbuf[i] = color_type;
/* Move along the longer dimension so that no pixels along the shorter 
   dimension are missed.  For now, the center of each pixel is the reference
   point, hence the 0.5 below.  A more sophisticated treatment should take
   the fractional part from the floating point input arguments in both x
   and y. */
  delta = 0.5;
  if (nvals_x >= nvals_y && nvals_y  != 1) {
    ratio = (float)(y2 - y1) / (float)(x2 - x1);
    if (ratio < 0.0)
      delta = -delta;
    if (x2 >= x1) {
      xstart = x1;
      ystart = y1;
/* Apply a nudge to the delta in order to produce symmetric looking vectors.
   This case arises when both ratio and delta are the same value, and is
   apparent when drawing circles.  */
      delta = delta +0.0001;
    } else {
      xstart = x2;
      ystart = y2;
      delta = delta -0.0001;
    }
    for (i = 0; i < nvals_x; i++) {
      startbuf = (short *)buffer.filebuf + (ystart * img.filecols) 
      + (int)((i * ratio) + delta) * img.filecols + xstart + i;
      expand_byte_to_short ((unsigned char *)readbuf, startbuf, 1);
    }
  } else if (nvals_y > nvals_x && nvals_x  != 1) {    
    ratio = (float)(x2 - x1) / (float)(y2 - y1);
    if (ratio < 0.0)
      delta = -delta;
    if (y2 >= y1) {
      ystart = y1;
      xstart = x1;
      delta = delta -0.0001;
    } else {
      ystart = y2;
      xstart = x2;
      delta = delta +0.0001;
    }
    for (i = 0; i < nvals_y; i++) {
      startbuf = (short *)buffer.filebuf + (ystart + i) * img.filecols
       + xstart + (int)((i * ratio) + delta);
      expand_byte_to_short ((unsigned char *)readbuf, startbuf, 1);
    }
  } else if (nvals_x > 1 && nvals_y == 1) {
      startbuf = (short *)buffer.filebuf + (y1 * img.filecols) + MIN(x1, x2);
      expand_byte_to_short ((unsigned char *)readbuf, startbuf, nvals_x);
  } else if (nvals_y > 1 && nvals_x == 1) {
      ystart = MIN(y1, y2);
      for (i = 0; i < nvals_y; i++) {
        startbuf = (short *)buffer.filebuf + ((ystart + i) * img.filecols)
        + x1;
        expand_byte_to_short ((unsigned char *)readbuf, startbuf, 1);
      }
  } else if (nvals_x == 1 && nvals_y == 1) {
      startbuf = (short *)buffer.filebuf + (y1 * img.filecols) + x1;
      expand_byte_to_short ((unsigned char *)readbuf, startbuf, 1);
  }
/*  If single vectors are being drawn, then key should be equal to 1 and
 *  this subroutine will take care of the call to disp_subpiece.  If a
 *  series of vectors is being drawn as in a circle, then key should be
 *  equal to 0 in the call and the calling probram should call disp_subpiece
 *  as is done in draw_on_frame.
*/
  if (key == 1)
  {
    disp_subpiece(0, MIN(y1,y2), img.filecols - 1, MAX(y1,y2)); 
  }
}

/*
 * Subroutine:	display_vect_on_frame()
 * Purpose:  This subroutine displays previously drawn vectors drawn
 * with vect_on_frame and key not equal to 1.
 */
void display_vect_on_frame ()
{
  short *imagebuf;
  void disp_subpiece();

    disp_subpiece(0, 0, img.filecols - 1, img.filecols - 1); 
}

/*
 * Subroutine:	expand_byte_to_short
 * Purpose:	Unpacks byte data for use as shorts.  Taken from irafio.c.
 */
static void expand_byte_to_short ( bts, ints, cnt )
     register unsigned char *bts;
     register short *ints;
     register int cnt;
{
  while (cnt-- > 0) {
    *ints = *bts;
    ints++;
    bts++;
  }
}
#endif
