/* Module:	ieeeunix.c (IEEE to Unix)
 * Purpose:	Converts an IEEE floating point array to Unix floating point
 * Subroutines:	ieee_unix()			returns:  void
 *		ieee_unix64()			returns:  void
 * Called by:	read_array in Readarr.c
 * Modified:	{0} Donald H. Gudehus;  initial version adapted from
 *		IEEEFLT by Patrick Murphy                    26 March, 2000
 *		{1} Donald H. Gudehus;  added ieee_unix64    31 May, 2005
 *		{n} <who> -- <does what> -- <when>
 */

union fstruc {
  unsigned long int longword;
  short int words[2];			/* for swapping */
  char bytes[4];			/* "   "        */
} ;

union fstruc64 {
  unsigned long int longword[2];
  short int words[4];                 /* for swapping */
  char bytes[8];                      /* "   "        */
} ;

void ieee_unix (buf, vals, bswap, wswap)
  unsigned long int *buf;
  int vals, bswap, wswap;

{
  float temp;
  int	i;
  short int itemp;		/* for swapping words */
  char ctemp;			/* for swapping bytes */
  union fstruc ieee;

  if (vals < 1) 
    return;
  for (i=0; i<vals; i++) {
    ieee.longword = buf[i];	/* Uses int (flt can cause fault) */
    if (bswap == 1) {
      ctemp = ieee.bytes[0];		/* byte swap (4 bytes) */
      ieee.bytes[0] = ieee.bytes[1];
      ieee.bytes[1] = ctemp;
      ctemp = ieee.bytes[2];
      ieee.bytes[2] = ieee.bytes[3];
      ieee.bytes[3] = ctemp;
    }
    if (wswap == 1) {
      itemp = ieee.words[0];		/* word swap */
      ieee.words[0] = ieee.words[1];
      ieee.words[1] = itemp;
    }

    buf[i] = ieee.longword;		/* puts Unix value back */
  }
}

void ieee_unix64 (buf,vals,bswap,wswap)
  unsigned long int *buf;
  int vals, bswap, wswap;

{
  int i;
  short int itemp;			/* for swapping words */
  long int ltemp;			/* for swapping long words */
  char ctemp;				/* for swapping bytes */
  union fstruc64 ieee64;

  if (vals < 1)
    return;
  for (i=0; i<vals*2; i=i+2) {
    ieee64.longword[0] = buf[i];
    ieee64.longword[1] = buf[i+1];
    if (bswap == 1) {
      ctemp = ieee64.bytes[0];		/* byte swap (4 bytes) */
      ieee64.bytes[0] = ieee64.bytes[1];
      ieee64.bytes[1] = ctemp;
      ctemp = ieee64.bytes[2];
      ieee64.bytes[2] = ieee64.bytes[3];
      ieee64.bytes[3] = ctemp;
      ctemp = ieee64.bytes[4];
      ieee64.bytes[4] = ieee64.bytes[5];
      ieee64.bytes[5] = ctemp;
      ctemp = ieee64.bytes[6];
      ieee64.bytes[6] = ieee64.bytes[7];
      ieee64.bytes[7] = ctemp;
    }
    if (wswap == 1) {
      itemp = ieee64.words[0];		/* word swap */
      ieee64.words[0] = ieee64.words[1];
      ieee64.words[1] = itemp;
      itemp = ieee64.words[2];
      ieee64.words[2] = ieee64.words[3];
      ieee64.words[3] = itemp;
      ltemp = ieee64.longword[0];
      ieee64.longword[0]=ieee64.longword[1];
      ieee64.longword[1]=ltemp;
    }
    buf[i] = ieee64.longword[0];	/* puts Unix value back */
    buf[i+1] = ieee64.longword[1];
  }
}
