/* Module:	ieeevax.c (IEEE to VAX)
 * Purpose:	Converts an IEEE floating point array to VAX floating point
 * Subroutines:	ieee_vax()			returns:  void
 *		ieee_vax64()			returns:  void
 * Called by:	read_array in Readarr.c
 * Modified:	{0} Donald H. Gudehus;	initial version adapted from
 *		IEEEFLT by Patrick Murphy		     7 July, 1993
 *		{1} Donald H. Gudehus;  added ieee_vax64     4 June, 2005
 *		{n} <who> -- <does what> -- <when>
 */
#define INDEF 1.7e38		/* or whatever you want it to be */

#include <stdio.h>
#include <stdlib.h>
#include <math.h>

union fstruc {
  unsigned long int longword;
  short int words[2];			/* for swapping */
  unsigned char bytes[4];		/* "   "        */
  float flt;	       			/* raw number */
  struct {				/* disassemble floating # */
    unsigned int mant1:7;		/* top part of fraction */
    unsigned int exponent:8;		/* Exponent with bias */
    unsigned int sign:1;		/* sign bit */
    unsigned int mant2:16;		/* other part of fraction */
  } f;
} ;

union fstruc64 {
  unsigned long int longword[2];
  short int words[4];			/* for swapping */
  unsigned char bytes[8];		/* "   "        */
  double flt;	       			/* raw number */
} ;


void ieee_vax (buf, vals )
  int vals;
  unsigned long int *buf;

{
  int	i;
  char ctemp;			/* for swapping bytes */
  union fstruc ieee;

  int sign,exponent;
  float fraction,answer;

  if (vals < 1) 
    return;
  for (i=0; i<vals; i++) {
    ieee.longword = buf[i];		/* Uses int (flt can cause fault) */

/*
  Alternate way of making the conversion by extracting the sign, exponent,
  and mantissa and then creating the floating point value.
*/
/*
    sign=pow(-1,(ieee.bytes[0]>>7));
    exponent=2*(ieee.bytes[0] & 0X7F) + (ieee.bytes[1]>>7) - 127;
    fraction=1.0 + (ldexp(1.0,-7))*(ieee.bytes[1] & 0X7F) +
    (ldexp(1.0,-15))*(ieee.bytes[2]) + (ldexp(1.0,-23))*(ieee.bytes[3]);
    answer=fraction*sign*ldexp(1.0,exponent);
*/
    ctemp = ieee.bytes[0];		/* byte swap (4 bytes) */
    ieee.bytes[0] = ieee.bytes[1];
    ieee.bytes[1] = ctemp;
    ctemp = ieee.bytes[2];
    ieee.bytes[2] = ieee.bytes[3];
    ieee.bytes[3] = ctemp;

/* let's check for NaN, Infinity, negative zero... */

    if (ieee.f.exponent == 255) 	/* NaN or Infinity. */
      ieee.flt = INDEF;			/*  don't care about sign */
    else if (ieee.f.exponent == 0)	/* Zap negative zero, causes */
      if ((ieee.f.mant1 == 0) &&	/*  reserved operand fault on */
      (ieee.f.mant2 == 0))		/*  a VAX (i.e. disregard */
        ieee.flt = 0.0;			/*  sign on +/- zero) */
      else				/* Denormalized IEEE number */
        ieee.flt = ieee.flt * 2;	/*  has exponent of -126 (??) */
    else {				/* Regular number */
      if (ieee.f.exponent >= 254)	/*  makes sure exponent won't */
        ieee.flt = INDEF;		/*  overflow */
      else {				/*  adds 2 to exponent */
        ieee.flt = ieee.flt * 4;	/*  (VAX has different bias) */
      }
    }
    buf[i] = ieee.longword;		/* puts VAX value back */
  }
}


void ieee_vax64(buf, vals )
  int vals;
  unsigned long int *buf;

{
  int i;
  union fstruc64 ieee64;

  int sign,exponent;
  double fraction,answer;

  if (vals < 1) 
    return;
  for (i=0; i<vals*2; i=i+2) {
    ieee64.longword[0] = buf[i];	/* Uses int (flt can cause fault) */
    ieee64.longword[1] = buf[i+1];

    sign=pow(-1,(ieee64.bytes[0]>>7));
    exponent=16*(ieee64.bytes[0] & 0X7F) + (ieee64.bytes[1]>>4) - 1023;
    fraction=1.0 + (ldexp(1.0,-4))*(ieee64.bytes[1] & 0XF) +
    (ldexp(1.0,-12))*(ieee64.bytes[2]) + (ldexp(1.0,-20))*(ieee64.bytes[3]) +
    (ldexp(1.0,-28))*(ieee64.bytes[4]) + (ldexp(1.0,-36))*(ieee64.bytes[5]) +
    (ldexp(1.0,-44))*(ieee64.bytes[6]) + (ldexp(1.0,-52))*(ieee64.bytes[7]);
    answer=fraction*sign*ldexp(1.0,exponent);
    ieee64.flt=answer;

    buf[i] = ieee64.longword[0];		/* puts VAX value back */
    buf[i+1] = ieee64.longword[1];
  }
}
