#ifndef lint
static char SccsId[] = "%W%  %G%";
#endif

/* Module:	imtlovrly.c (Imtool Overlay)
 * Purpose:	Caries out tasks related to drawing overlays (erasable and
 * nonerasble cursor and vector objects) when supplied with instructions 
 * via the Imtool protocol.  
 *
 * Subroutine:	do_overlay_from_imtool ()	returns:  number of remaining
 *					        bytes to be read
 * Modified:	{0} Initial version, Donald H. Gudehus	    18 August, 1993
 *		{1} Donald Gudehus, introduced cases of action=3, 4, and 5 
 *		vectors on frame buffer			 28 September, 1994
 *		{2} Donald Gudehus, allowed for arbitray number of annuli,
 *							    3 October, 1994
 *		{n} <who> -- <does what> -- <when>
 */

#ifdef IMTOOL

#include <stdio.h>		/* stderr, FILE, NULL, etc. */
#include <X11/Xlib.h>		/* gets X types and constants */
#include <X11/Xutil.h>		/* X window manager stuff */
#include "hfiles/constant.h"	/* defines codes */
#include "hfiles/struct.h"	/* declares structure types */
#include "hfiles/extern.h"	/* extern main ximage parameter structures */
#include "hfiles/imtool.h"
#include "hfiles/region.h"	/* regdrawRec region drawing parameters */
#include <math.h>
#define SZ_IMCURVAL     160
struct regdrawRec rgdraw;

/*
 * Subroutine:	do_overlay_from_imtool ()
 * Purpose:	Interprets the data meant for drawing and erasing overlays 
 * sent via the Imtool protocol, and carries out the operations.  The value
 * of action determines the operation.  If 0, cursors and annuli are erased,
 * if 10, regions are erased as well, if 1, cursors are drawn, if 11, 
 * cursors are drawn as regions, if 2, cursors are written into the frame
 * buffer.  In this latter case, the entire frame buffer pixel is lit up, and
 * the cursors are not erasable unless the frame is erased.  The operation
 * is more fully explained in SAO_DRAW.FOR.  The x and y coordinates of 
 * the center of an overlay object are measured in the file system
 * coordinates.  If several files have been loaded into the frame, it is 
 * up to the connecting image processing program to calculate the positions 
 * referenced to the file most recently loaded.
 */
int do_overlay_from_imtool ( port, ndatabytes)
     struct connectRec *port;	/* i:  struct of port for Imtool input */
     int ndatabytes;		/* i:  number of bytes in expected message */
{
  int read_connection();
  void d_transform(), draw_magnifier(), disp_region();
  void set_cursor_from_file_coords();
  void label_region_cycle_magnifier(), save_cursor_as_region(), erase_cursor();
  void disp_dispbox(), draw_cursor(), make_new_annulus(), draw_annuli();
  void disp_panbox(), set_polygon_from_file_coords();

  union {
    char buf[SZ_IMCURVAL];
    float extra[SZ_IMCURVAL/4];
  } udo;
  float file_X, file_Y, a, b, inclina, ratio;
  float bufX, bufY;
  int bytes;
  int tokens;                   /*  number of tokens successfully parsed */
  int action, cursor_type, color_type, key, exclude, nextra, i ,ii, hash;
  unsigned long background;     /* l: temp for switch of colors */

/* Read the byte stream which is sent following the header */
  bytes = read_connection(port, udo.buf, ndatabytes);
/* Compute the number of remaining bytes to be read, if things don't 
   work out (unlikely) */
  ndatabytes -= bytes;
  tokens = sscanf(udo.buf, "%d%d%d%f%f%f%f%f", &action, &cursor_type,
  &color_type, &file_X, &file_Y, &a, &b, &inclina);
  if (bytes > 67) {  /*  The number of bytes is set up in SAO_DRAW.FOR */
    nextra = (bytes - 67)/10;
    ii = 0;
    for (i = 68; i < bytes; i = i + 10) {
      sscanf(&udo.buf[i], "%f", &udo.extra[ii]);
      ii = ii +1;
    }
  } else {
    nextra = 0;
  }
  if ( action == 0 || action == 10 ) {  /* Erases cursors, annuli, and 
					   regions*/
    if ( action == 10)
      reset_regions();
    if (cursor.annuli) {
      delete_annuli(&cursor, 1);
      cursor.annuli = 0;
    }
    erase_cursor(&cursor);
/* Draw a point cursor and change the color to green */
    cursor.file.X = file_X;
    cursor.file.Y = file_Y;
    cursor.type = 256;
    cursor.file.Xdim = 4.0;
    cursor.file.Ydim = 4.0;
    color.gcset.draw.foreground =color.hard.green;
    set_cursor_from_file_coords(&cursor, &coord.filetodisp);
    disp_dispbox();
    disp_panbox();
    touch_submenu_button(COP, COP_Point);
    return ( ndatabytes);

  } else if ( action == 1 || action == 11) {  /* Draws cursor of selected 
					      color and type */
    cursor.file.X = file_X;
    cursor.file.Y = file_Y;
    cursor.type = cursor_type;
    cursor.file.Xdim = a;
    cursor.file.Ydim = b;
    inclina = inclina / 57.29577951; /* inclina is ccw and in degrees;  internal
                                  angle is in radians and is cw */
    cursor.rot.angle = inclina;
    cursor.rot.cos = cos(-(double)inclina);
    cursor.rot.sin = sin(-(double)inclina);
    touch_submenu_button (COP, cursor.type); 
/* Take care of what's needed to do for each cursor type */
    if (color_type == 201)
      color.gcset.draw.foreground =color.hard.green;
    if (color_type == 202)
      color.gcset.draw.foreground =color.hard.true_black;
    if (color_type == 203)
      color.gcset.draw.foreground =color.hard.std_white;
    if (color_type == 204)
      color.gcset.draw.foreground =color.hard.red;
    if (color_type == 206)
      color.gcset.draw.foreground =color.hard.blue;
    if (color_type == 207)
      color.gcset.draw.foreground =color.hard.yellow;
/* Display the requested cursor */
    if ( nextra == 0) {
      cursor.annuli = 0;
      set_cursor_from_file_coords(&cursor, &coord.filetodisp);
/* Take care of any annuli which might happen to be in string */
    } else if (nextra >= 1 && cursor.type != COP_Polygon) {
      cursor.annuli = 1;
      ratio = b/a;
      set_cursor_from_file_coords(&cursor, &coord.filetodisp);
      make_new_annulus(&cursor);
      for (i = 0; i < nextra; i++) {
        cursor.file.Xdim = udo.extra[i];
        cursor.file.Ydim = udo.extra[i]*ratio;
        set_cursor_from_file_coords(&cursor, &coord.filetodisp);
        make_new_annulus(&cursor);
      }
      draw_annuli(&cursor, cursor.draw);
    } else if (nextra >=1 && cursor.type == COP_Polygon) {
/*  Not ready for prime time */
      hash = 0;
      cursor.poly_cnt = nextra + 1;
      for (i = 0; i < nextra; i = i + 2) {
/*        cursor.poly[i].fileX = udo.extra[i];
        cursor.poly[i].fileY = udo.extra[i + 1];  not right */
      }
      set_polygon_from_file_coords(&cursor, &coord.filetodisp, hash);
    }
/* For action = 11, save the cursor(s) as a region */
    if (action == 11) {
      exclude = 0;
      set_cursor_file_coords(&cursor, &coord.disptofile, 0);
      set_cursor_file_coords(&cursor, &coord.disptofile, 1);
      save_cursor_as_region(&cursor, exclude);
/*      rgdraw.label = 0;   Prevents label from being drawn */
      disp_region(cursor.next_region);
    }
  /* show cursor position in magnifier, including coordinate label */
    d_transform(&coord.disptobuf, cursor.win.X, cursor.win.Y, &bufX, &bufY);
    draw_magnifier ((double)bufX, (double)bufY);
    background = cursor.draw->background;
    cursor.draw->background = color.gcset.excl.background;
    label_region_cycle_magnifier (&cursor, 0);
    cursor.draw->background = background; 

    disp_panbox();
    disp_dispbox();
    /*  draw_cursor(&cursor, draw);  */
  } else if ( action == 2) {  /* Draws the cursor object on the screen by 
                                 writing into the buffer */
    draw_on_frame (cursor_type, color_type, file_X, file_Y, a, b, inclina);
  } else if ( action == 3) {  
       /* Draws a vector on the screen by writing into the buffer.  In
          this case, file_X  and file_Y are the starting coordinates in
          the file, and a and b are the ending coordinates in the file. 
	  The vector is immediately displayed  by calling disp_subpiece. 
       */
    key =1;
    vect_on_frame ( color_type, file_X, file_Y, a, b, key);
  } else if ( action == 4) {  
       /* Draws a vector on the screen by writing into the buffer as
	  in action=3, but the vector is not immediately displayed.  A
	  call with action=5 will later need to be made in order to
	  display it.  This saves time when many vectors are being
	  drawn.
       */
    key =0;
    vect_on_frame ( color_type, file_X, file_Y, a, b, key);
  } else if ( action == 5) {  
    display_vect_on_frame ();
  }
  return ( ndatabytes );
}

#endif
